import numpy as np
import pandas as pd
from sys import path

# import ROOT

from matplotlib.backends.backend_pdf import PdfPages
import matplotlib.pyplot as plt

# path.append("@CONFIG_PY_DIR@")
path.append("/home/sutclqw/Work/Analysis/Hammer-install/lib/python3.8/site-packages")

from hammer.hammerlib import Hammer, IOBuffer, Particle, Process, FourMomentum
from hammer import hepmc, pdg

HAMMER = Hammer()
BUFFER = IOBuffer()
PROC = Process()

fig, ax = plt.subplots()
figtaucms, axtaucms = plt.subplots()
figtauBrest, axtauBrest = plt.subplots()
figmm2, axmm2 = plt.subplots()
fignum2, axnum2 = plt.subplots()
fignuE, axnuE = plt.subplots()
fignupx, axnupx = plt.subplots()
fignupy, axnupy = plt.subplots()
fignupz, axnupz = plt.subplots()

figmm2reco, axmm2reco = plt.subplots()
figpBrestreco, axpBrestreco = plt.subplots()
figwReco, axwReco = plt.subplots()


def boostVec(tpE, tpx, tpy, tpz, bpE, bpx, bpy, bpz, sE, spx, spy, spz):
    # Normalize the boosted vector
    E = tpE
    px = tpx  # /tpE
    py = tpy  # /tpE
    pz = tpz  # /tpE

    # get the boost vector from the actual 4-vector
    bx = -bpx / bpE
    by = -bpy / bpE
    bz = -bpz / bpE

    b2 = bx * bx + by * by + bz * bz
    gamma = 1.0 / (np.sqrt(1.0 - b2))
    bp = (bx * px) + (by * py) + (bz * pz)
    gamma2 = 0
    if b2 > 0:
        gamma2 = (gamma - 1.0) / b2
    else:
        gamma2 = 0
    return pd.Series(
        [
            (gamma * (E + bp)),
            (px + gamma2 * bp * bx + gamma * bx * E),
            (py + gamma2 * bp * by + gamma * by * E),
            (pz + gamma2 * bp * bz + gamma * bz * E),
        ],
        index=[sE, spx, spy, spz],
    )


def procEvent(
    b1pdg,
    b1E,
    b1px,
    b1py,
    b1pz,
    b1dspdg,
    b1dsE,
    b1dspx,
    b1dspy,
    b1dspz,
    b1dsd1pdg,
    b1dsd1E,
    b1dsd1px,
    b1dsd1py,
    b1dsd1pz,
    b1dsd2pdg,
    b1dsd2E,
    b1dsd2px,
    b1dsd2py,
    b1dsd2pz,
    b1tpdg,
    b1tE,
    b1tpx,
    b1tpy,
    b1tpz,
    b1td1pdg,
    b1td1E,
    b1td1px,
    b1td1py,
    b1td1pz,
    b1td2pdg,
    b1td2E,
    b1td2px,
    b1td2py,
    b1td2pz,
    b1td3pdg,
    b1td3E,
    b1td3px,
    b1td3py,
    b1td3pz,
    b1nupdg,
    b1nuE,
    b1nupx,
    b1nupy,
    b1nupz,
    q2,
    pl,
    mm2,
    weight,
):
    #    if((abs(int(b1pdg))==pdg.BPLUS or abs(int(b1pdg))==pdg.BZERO) and (abs(int(b1dspdg)==pdg.DSTAR) or abs(int(b1dspdg))==pdg.DSTARPLUS) and abs(int(b1tpdg))==pdg.TAU):
    ids = set()
    if (
        (abs(int(b1pdg)) == 511 or abs(int(b1pdg)) == 521)
        and (abs(int(b1dspdg) == 413) or abs(int(b1dspdg)) == 423)
        and abs(int(b1tpdg)) == 15
    ):
        #        print(str(b1pdg) + ' -> ' + str(b1dspdg) + ' ' + str(b1tpdg) + ' '+ str(b1nupdg))
        HAMMER.init_event()
        PROC = Process()
        BID = PROC.add_particle(Particle(FourMomentum(b1E, b1px, b1py, b1pz), b1pdg))
        DsID = PROC.add_particle(Particle(FourMomentum(b1dsE, b1dspx, b1dspy, b1dspz), b1dspdg))
        DsD1ID = PROC.add_particle(Particle(FourMomentum(b1dsd1E, b1dsd1px, b1dsd1py, b1dsd1pz), b1dsd1pdg))
        DsD2ID = PROC.add_particle(Particle(FourMomentum(b1dsd2E, b1dsd2px, b1dsd2py, b1dsd2pz), b1dsd2pdg))
        # DsD2ID=PROC.add_particle(Particle(FourMomentum((b1dsE-b1dsd1E), (b1dspx-b1dsd2px), (b1dspy-b1dsd2py), (b1dspz-b1dsd2pz)),b1dsd2pdg))
        tauID = PROC.add_particle(Particle(FourMomentum(b1tE, b1tpx, b1tpy, b1tpz), b1tpdg))
        taudaugs = []
        if b1td3pdg != -999 and (abs(b1td1pdg) == 11 or abs(b1td1pdg) == 13):
            emom = Particle(FourMomentum(b1td1E, b1td1px, b1td1py, b1td1pz), b1td1pdg)
            nutaumom = Particle(FourMomentum(b1td2E, b1td2px, b1td2py, b1td2pz), b1td3pdg)
            nuemom = Particle(FourMomentum(b1td3E, b1td3px, b1td3py, b1td3pz), b1td2pdg)

            d4px = b1tpx - b1td1px - b1td2px - b1td3px
            d4py = b1tpy - b1td1py - b1td2py - b1td3py
            d4pz = b1tpz - b1td1pz - b1td2pz - b1td3pz
            d4E = np.sqrt(d4px * d4px + d4py * d4py + d4pz * d4pz)

            depx = b1tpx - b1td2px - b1td3px
            depy = b1tpy - b1td2py - b1td3py
            depz = b1tpz - b1td2pz - b1td3pz
            deE = np.sqrt(depx * depx + depy * depy + depz * depz)

            emommod = Particle(FourMomentum(deE, depx, depy, depz), b1td1pdg)

            tauD1ID = PROC.add_particle(emommod)
            tauD2ID = PROC.add_particle(nutaumom)
            tauD3ID = PROC.add_particle(nuemom)
            # print(str(b1tpdg) + " -> " + str(b1td1pdg) + " + " + str(b1td2pdg) + " + " + str(b1td3pdg))
            # print( str(emom.p.mass()) + " + " + str(nutaumom.p.mass()) + " + " + str(nuemom.p.mass()))

            # if(d4E>0.001):
            #    tauD4ID=PROC.add_particle(Particle(FourMomentum(d4E, d4px, d4py, d4pz),22))
            #    taudaugs=[tauD1ID, tauD2ID, tauD3ID, tauD4ID]
            # else:
            taudaugs = [tauD1ID, tauD2ID, tauD3ID]
        else:
            tauD1ID = PROC.add_particle(Particle(FourMomentum(b1td1E, b1td1px, b1td1py, b1td1pz), b1td1pdg))
            tauD2ID = PROC.add_particle(Particle(FourMomentum(b1td2E, b1td2px, b1td2py, b1td2pz), b1td2pdg))
            taudaugs = [tauD1ID, tauD2ID]
        nuID = PROC.add_particle(Particle(FourMomentum(b1nuE, b1nupx, b1nupy, b1nupz), b1nupdg))
        # nuID=PROC.add_particle(Particle(FourMomentum((b1E-b1dsE-b1tE), (b1px-b1dspx-b1tpx), (b1py-b1dspy-b1tpy), (b1pz-b1dspz-b1tpz)),b1nupdg))
        Bdaugs = [DsID, tauID, nuID]
        Dsdaugs = [DsD1ID, DsD2ID]
        PROC.add_vertex(BID, Bdaugs)
        PROC.add_vertex(DsID, Dsdaugs)
        PROC.add_vertex(tauID, taudaugs)
        procid = HAMMER.add_process(PROC)
        if procid != 0:
            ids.add(procid)

        # if(pl>0 and q2>0):
        # HAMMER.set_event_base_weight(weight*2.5)
        HAMMER.fill_event_histogram("pEllVsQ2:D*", [pl, q2])
        HAMMER.fill_event_histogram("pEllVsMM2:D*", [pl, mm2])
        #       #second B fed to HAMMER (not possible atm)
        if len(ids) > 0:
            HAMMER.process_event()
            ffweight = HAMMER.get_weight("Scheme1")
            if np.isnan(ffweight):
                print("\n\nWEIGHT IS FUCKED UP\n\n")
                print(
                    [
                        (b1tE - b1td1E - b1td2E - b1td3E),
                        (b1tpx - b1td1px - b1td2px - b1td3px),
                        (b1tpy - b1td1py - b1td2py - b1td3py),
                        (b1tpz - b1td1pz - b1td2pz - b1td3pz),
                    ]
                )

        #        #whatever else we want to do with HAMMER at event level~
        return 1

    return 0


# Read in the dataframes
dfpath = "./signal_selection_mc_main/"
df = pd.read_hdf(dfpath + "Mixed_withFinalWeights_optimized_mva_selection_sec_sel_mva_with_CS_CS1.h5")
q1 = df["daughter__bo1__cm__spextraInfo__bodecayModeID__bc__bc"]
q2 = df["SIG_ID"]
dfcut = df[((q1 == 15) | (q1 == 16)) & (q2 == 2)]

df_dic = {
    # "charged": pd.read_hdf(dfpath+'Charged_withFinalWeights_optimized_mva_selection_sec_sel_mva_with_CS_CS1.h5'),
    # "charm": pd.read_hdf(dfpath+'Charm_withFinalWeights_first_selection_with_CS_CS1_full_q2.h5'),
    # "dss": pd.read_hdf(dfpath+'Dss_withFinalWeights_first_selection_with_CS_CS1_full_q2.h5'),
    # "gap": pd.read_hdf(dfpath+'Gap_withFinalWeights_first_selection_with_CS_CS1_full_q2.h5'),
    "mixed": dfcut,
    # "specialpsi": pd.read_hdf(dfpath+'SpecialPsi_withFinalWeights_first_selection_with_CS_CS1_full_q2.h5'),
    # "special": pd.read_hdf(dfpath+'Special_withFinalWeights_first_selection_with_CS_CS1_full_q2.h5'),
    # "uds": pd.read_hdf(dfpath+'UDS_withFinalWeights_first_selection_with_CS_CS1_full_q2.h5')
}
print("Loaded dataframes.")


# Set the hammer options
HAMMER.include_decay(["BD*TauNu", "TauEllNuNu"])
# HAMMER.include_decay(["BD*TauNu"])
HAMMER.add_ff_scheme("Scheme2", {"BD*": "BGL"})
HAMMER.add_ff_scheme("Scheme1", {"BD*": "BLPR"})
HAMMER.add_ff_scheme("Scheme0", {"BD*": "ISGW2"})
HAMMER.set_ff_input_scheme({"BD*": "ISGW2"})
HAMMER.add_histogram("pEllVsQ2:D*", [8, 7], False, [[0.25, 2.25], [3.0, 12.0]])
HAMMER.add_histogram("pEllVsMM2:D*", [8, 7], False, [[0.25, 2.25], [-2.0, 5.0]])
HAMMER.keep_errors_in_histogram("pEllVsQ2:D*", True)
HAMMER.keep_errors_in_histogram("pEllVsMM2:D*", True)
HAMMER.collapse_processes_in_histogram("pEllVsQ2:D*")
HAMMER.collapse_processes_in_histogram("pEllVsMM2:D*")
HAMMER.add_total_sum_of_weights()
HAMMER.set_units("GeV")
HAMMER.init_run()


legendkeys = []
# h1=ROOT.TH1D('ptaucms','ptaucms',50,0,2.5)
# Loop over all types
for types in df_dic.keys():
    print("Opening file " + str(types))
    df = df_dic[types]
    df_tau = df[
        (df["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmPDG__bc__bc__bc"].abs() == 15)
        & (
            (df["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmPDG__bc__bc__bc"].abs() == 413)
            | (df["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmPDG__bc__bc__bc"].abs() == 423)
        )
        & (
            (df["genUpsilon4S__bomcDaughter__bo1__cm__spPDG__bc__bc"].abs() == 511)
            | (df["genUpsilon4S__bomcDaughter__bo1__cm__spPDG__bc__bc"].abs() == 521)
        )
    ]
    if len(df_tau) != 0:
        print("template " + str(types) + " has " + str(len(df_tau)) + " entries")
        # if(str(types)!='charged'): continue
        df_tau.loc[:, "genMC_processed"] = df_tau.apply(
            lambda row: procEvent(
                row["genUpsilon4S__bomcDaughter__bo1__cm__spPDG__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spE__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__sppx__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__sppy__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__sppz__bc__bc"],  # second B infos
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmPDG__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmE__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmpx__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmpy__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmpz__bc__bc__bc"],  # Dstar infos
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmE__bc__bc__bc__bc"],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpx__bc__bc__bc__bc"
                ],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpy__bc__bc__bc__bc"
                ],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpz__bc__bc__bc__bc"
                ],  # first Dstar daughter infos
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmmcDaughter__bo1__cmPDG__bc__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmE__bc__bc__bc__bc"],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpx__bc__bc__bc__bc"
                ],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpy__bc__bc__bc__bc"
                ],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpz__bc__bc__bc__bc"
                ],  # second Dstar daughter infos
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmPDG__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmE__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc"],  # tau lepton infos
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmE__bc__bc__bc__bc"],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpx__bc__bc__bc__bc"
                ],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpy__bc__bc__bc__bc"
                ],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpz__bc__bc__bc__bc"
                ],  # first tau lepton daughter info
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmmcDaughter__bo1__cmPDG__bc__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmE__bc__bc__bc__bc"],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc__bc"
                ],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc__bc"
                ],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc__bc"
                ],  # second tau lepton daughter info
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmmcDaughter__bo2__cmPDG__bc__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo2__cmE__bc__bc__bc__bc"],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo2__cmpx__bc__bc__bc__bc"
                ],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo2__cmpy__bc__bc__bc__bc"
                ],
                row[
                    "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo2__cmpz__bc__bc__bc__bc"
                ],  # third tau lepton daughter info
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmPDG__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmE__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpx__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpy__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpz__bc__bc__bc"],  # tau neutrino info
                row["qSquaredReco"],
                row["pStarL"],
                row["m2RecoilSignalSide"],
                row["__new_weight_final_wBF_corr__"],
            ),
            axis=1,
        )

        legendkeys.append(str(types))

        series = df_tau.apply(
            lambda row: boostVec(
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmE__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc"],  # tau lepton infos
                11.5,
                0,
                0,
                4.5,  # Boost infos
                "genUpsilon4S_tauEcms",
                "genUpsilon4S_taupxcms",
                "genUpsilon4S_taupycms",
                "genUpsilon4S_taupzcms",
            ),
            axis=1,
        )  # names for the new columns

        series.name = "tauCMS"

        df_tau = pd.concat([df_tau, series], axis=1)

        series2 = df_tau.apply(
            lambda row: boostVec(
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmE__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc"],  # tau lepton infos
                row["genUpsilon4S__bomcDaughter__bo1__cm__spE__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__sppx__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__sppy__bc__bc"],
                row["genUpsilon4S__bomcDaughter__bo1__cm__sppz__bc__bc"],  # Boost infos
                "genUpsilon4S_tauEBrest",
                "genUpsilon4S_taupxBrest",
                "genUpsilon4S_taupyBrest",
                "genUpsilon4S_taupzBrest",
            ),
            axis=1,
        )  # names for the new columns

        series2.name = "tauBrest"
        df_tau = pd.concat([df_tau, series2], axis=1)

        df_tau.loc[:, "genUpsilon4S_ptaucms"] = np.sqrt(
            df_tau["genUpsilon4S_taupxcms"] * df_tau["genUpsilon4S_taupxcms"]
            + df_tau["genUpsilon4S_taupycms"] * df_tau["genUpsilon4S_taupycms"]
            + df_tau["genUpsilon4S_taupzcms"] * df_tau["genUpsilon4S_taupzcms"]
        )
        df_tau.loc[:, "genUpsilon4S_ptauBrest"] = np.sqrt(
            df_tau["genUpsilon4S_taupxBrest"] * df_tau["genUpsilon4S_taupxBrest"]
            + df_tau["genUpsilon4S_taupyBrest"] * df_tau["genUpsilon4S_taupyBrest"]
            + df_tau["genUpsilon4S_taupzBrest"] * df_tau["genUpsilon4S_taupzBrest"]
        )

        df_tau.loc[:, "genUpsilon4S_ptau"] = np.sqrt(
            df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc"]
            * df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc"]
            + df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc"]
            * df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc"]
            + df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc"]
            * df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc"]
        )

        df_tau.loc[:, "genUpsilon4S_nuE"] = (
            df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spE__bc__bc"]
            - df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmE__bc__bc__bc__bc"]
            - df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmE__bc__bc__bc"]
        )
        df_tau.loc[:, "genUpsilon4S_nupx"] = (
            df_tau["genUpsilon4S__bomcDaughter__bo1__cm__sppx__bc__bc"]
            - df_tau[
                "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpx__bc__bc__bc__bc"
            ]
            - df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc"]
        )
        df_tau.loc[:, "genUpsilon4S_nupy"] = (
            df_tau["genUpsilon4S__bomcDaughter__bo1__cm__sppy__bc__bc"]
            - df_tau[
                "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpy__bc__bc__bc__bc"
            ]
            - df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc"]
        )
        df_tau.loc[:, "genUpsilon4S_nupz"] = (
            df_tau["genUpsilon4S__bomcDaughter__bo1__cm__sppz__bc__bc"]
            - df_tau[
                "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpz__bc__bc__bc__bc"
            ]
            - df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc"]
        )
        df_tau.loc[:, "genUpsilon4S_numm2"] = df_tau["genUpsilon4S_nuE"] * df_tau["genUpsilon4S_nuE"] - (
            df_tau["genUpsilon4S_nupx"] * df_tau["genUpsilon4S_nupx"]
            + df_tau["genUpsilon4S_nupy"] * df_tau["genUpsilon4S_nupy"]
            + df_tau["genUpsilon4S_nupz"] * df_tau["genUpsilon4S_nupz"]
        )

        df_tau.loc[:, "genUpsilon4S_num2"] = df_tau[
            "genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmE__bc__bc__bc"
        ] * df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmE__bc__bc__bc"] - (
            df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpx__bc__bc__bc"]
            * df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpx__bc__bc__bc"]
            + df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpy__bc__bc__bc"]
            * df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpy__bc__bc__bc"]
            + df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpz__bc__bc__bc"]
            * df_tau["genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpz__bc__bc__bc"]
        )

        df_tau.hist("qSquaredReco", stacked=True, ax=axwReco)
        df_tau.hist("pStarL", stacked=True, ax=axpBrestreco)
        df_tau.hist("mcMissingMassSquared", stacked=True, ax=axmm2reco)

with open("./PyHAMMER_Histos.dat", "wb") as fout:
    BUFFER = HAMMER.save_run_header()
    BUFFER.save(fout)
    BUFFER = HAMMER.save_histogram("Total Sum of Weights")
    BUFFER.save(fout)
    BUFFER = HAMMER.save_histogram("pEllVsQ2:D*")
    BUFFER.save(fout)
    BUFFER = HAMMER.save_histogram("pEllVsMM2:D*")
    BUFFER.save(fout)

axpBrestreco.legend(legendkeys)
axwReco.legend(legendkeys)
axmm2reco.legend(legendkeys)
figpBrestreco.savefig("pl_df.pdf")
figwReco.savefig("q2_df.pdf")
figmm2reco.savefig("mm2_df.pdf")
# Do the fit with Hammer etc.
